//
// $Id: basetrie.hh,v 1.1 2000/07/29 22:10:53 voeckler Exp $
//
// Author:  Jens-S. Vckler <voeckler@rvs.uni-hannover.de>
//
// File:    basetrie.hh
//          Fri Jul 28 2000
//
// (c) 2000 Lehrgebiet Rechnernetze und Verteilte Systeme
//          Universitt Hannover, Germany
//
// Permission to use, copy, modify, distribute, and sell this software
// and its documentation for any purpose is hereby granted without fee,
// provided that (i) the above copyright notices and this permission
// notice appear in all copies of the software and related documentation,
// and (ii) the names of the Lehrgebiet Rechnernetze und Verteilte
// Systeme and the University of Hannover may not be used in any
// advertising or publicity relating to the software without the
// specific, prior written permission of Lehrgebiet Rechnernetze und
// Verteilte Systeme and the University of Hannover.
//
// THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
// EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
// WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
//
// IN NO EVENT SHALL THE LEHRGEBIET RECHNERNETZE UND VERTEILTE SYSTEME OR
// THE UNIVERSITY OF HANNOVER BE LIABLE FOR ANY SPECIAL, INCIDENTAL,
// INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY DAMAGES
// WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER OR NOT
// ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF LIABILITY,
// ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
// SOFTWARE.
//
// $Log: basetrie.hh,v $
// Revision 1.1  2000/07/29 22:10:53  voeckler
// Initial revision
//
#ifndef _BASETRIE_HH
#define _BASETRIE_HH

#if defined(__GNUC__) || defined(__GNUG__)
#pragma interface
#endif

#include <sys/types.h>
#include "typedefs.h"

class BasicTrie {
  // abstract interface to trie classes
public:
  virtual ~BasicTrie() = 0;
  // purpose: abstract, empty dtor.

  virtual size_t size() const = 0;
  // returns: number of leaf nodes

  virtual size_t distinct() const = 0;
  // returns: number of distinct tokens: distinct() <= size()

  virtual 
  bool 
  insert( const char* word, MyUInt32 value ) = 0;
  // purpose: insert a new piece into the symbol table
  // paramtr: name (IN): new name to insert into symtab
  //          value (IN): token value to use for default override
  // returns: true, if insertion was successful.
  // returns: false, e.g. if there were illegal characters.
  // warning: double inserts will overwrite the old value

  virtual 
  const char* 
  reverse( char* buffer, size_t bsize, MyUInt32 value ) const = 0;
  // purpose: find the matching string to a given token value
  // paramtr: buffer (IO): area to store resulting string into
  //          bsize (IN): size of the area to store data into
  //          value (IN): value to look for
  // returns: a pointer to the buffer, which will contain
  //	      "<error>" if the value was not found.
  
  virtual 
  MyUInt32 
  find( const char* word ) const = 0;
  // purpose: search the trie for the given word, and return token value
  // paramtr: word (IN): word to search trie for
  // returns: 0 for not found, tokenvalue otherwise
  //          The return value of 0 lets you count the errors, too.
  // warning: PRECONDITION: word must not be null!
};

#endif // _BASETRIE_HH
