#! /sw/perl-5.005/bin/perl
#
# Please note that we may safely use @Apache::Registry::something, even
# if the script is run as regular CGI perl program and not as mod_perl.
#
require 5.005;
use strict;
use DBI;
use POSIX qw(:sys_wait_h :time_h);

use File::Basename;
use lib dirname($0);
use Scripts qw(parseQueryString now @mon);
use OldWrapper;

# FIXME: where is your gnuplot?!
my $gnuplot = '/sw/gnuplot/bin/gnuplot';
#my $gnuplot = '/opt/gnuplot/bin/gnuplot';

my $tmpdir = $ENV{TMP} || '/var/tmp';
my $maxage = 5;		# maximum cachability of output page
umask(002);

sub numerical ($) {
    # purpose: obtain single numerical parameter and untaint it
    my $x = shift;
    $x = $x->[ @{$x}-1 ] if ref $x;
    $x =~ tr/[0-9]//cs;
    $x+0;
}

# sanitize, see CERT warning
my %form = parseQueryString();
my $shour = numerical($form{shour});
my $syear = numerical($form{syear});
$syear -= 1900 if $syear > 1900;
my $sday  = numerical($form{sday});
my $smon  = numerical($form{smon});
my $start = mktime(0,0,$shour,$sday,$smon,$syear);

# parse finish time
my $fhour = numerical($form{fhour});
my $fyear = numerical($form{fyear});
$fyear -= 1900 if $fyear > 1900;
my $fday  = numerical($form{fday});
my $fmon  = numerical($form{fmon});
my $final = mktime(0,0,$fhour,$fday,$fmon,$fyear);

if ( ! exists $form{cache} ) {
    my $msg = "Sorry, but you have to activate at least one cache host.\n";
    print << "EOF";
Content-Type: text/plain; charset=iso-8859-1
Content-Length: @{[length($msg)]}
Expires: @{[&now]}
Pragma: no-cache
Cache-Control: private

$msg
EOF
    exit(1);
} 
my @cache = ref $form{cache} ? @{$form{cache}} : ($form{cache});
my $n = 0+@cache;
my $cacheset = "\'" . join("\',\'",@cache) . "\'";

my $type = numerical($form{type});
my @title = ( 'Hourly TCP and TCP HIT',
	      '!GET methods',
	      '*.win-ip.dfn.de as TCP client',
	      'IN, OUT and AMONG',
	      'Frequent toplevel domains',
	      'Frequent MIME types',
	      'Top N toplevel domains',
	      'Top N destination AS',
	      'Top N UDP clients',
	      'Top N UDP HIT clients',
	      'Top N TCP clients',
	      'Top N TCP HIT clients',
	      'Hierarchy performance'
	      );

my $kind = numerical($form{kind});
my @yaxis = ( 'Requests [1]', 
	      'Volume [Byte]' );

my @isize = ( '640,480', '800,600', '1024,768' );
my $isize = numerical($form{isize});
my $format = numerical($form{format});
my @suffix = qw(gif png eps);
my @ctype = ( 'image/gif', 
	      'image/png',
	      'application/postscript',
	      'application/octet-stream'
	      );
my $pid = $$;
my @result;
my @names;			# file names to unlink after work is done.
my $cf = "$tmpdir/$pid.cf";
my $of = "$cf.$suffix[$format]";
my @callya = ( \&ctor0,  \&ctor1,  \&ctor2,  \&ctor3,  \&ctor4,
	       \&ctor5,  \&ctor6,  \&ctor7,  \&ctor8,  \&ctor9,
	       \&ctor10, \&ctor11, \&ctor12 );

eval {
    &database_login;

    open( CF, ">$cf" ) || die "open($cf): $!";
    push @names, $cf; # unlink after exit
    if ( $format == 0 ) {
	# gif output
	print CF "set term gif interlace small size $isize[$isize]\n";
    } elsif ( $format == 1 ) {
	print CF "set term png small color\n";
	print CF "set size 1.44,1.44\n" if ( $isize == 1 );
	print CF "set size 2,2\n" if ( $isize == 2 );
    } elsif ( $format == 2 ) {
	print CF "set term post eps color solid \"Helvetica-Bold\" ";
	if ( $isize == 0 ) {
	    print CF "14\nset size 1.4,1.4\n";
	} elsif ( $isize == 1 ) {
	    print CF "18\nset size 2,2\n";
	} else {
	    print CF "24\nset size 3,3\n";
	}
    } else {
	print CF "set term aifm color \"Helvetica-Bold\" ";
	if ( $isize == 0 ) {
	    print CF "14\nset size 1.4,1.4\n";
	} elsif ( $isize == 1 ) {
	    print CF "18\nset size 2,2\n";
	} else {
	    print CF "24\nset size 3,3\n";
	}
    }
    print CF << "EOF";
set output "$of"
set grid
set xdata time
set timefmt "%s"
set format x "%d.%m"
set title "$title[$type]\\n$cacheset"
set ylabel "$yaxis[$kind]"
EOF
    
    if ( defined $callya[$type] ) {
	# no strict 'subs';
	my $cfg = &{$callya[$type]};
	print CF $cfg, "\n";
    } else {
	die "Sorry, you selected the query of type=$type" .
	    " which is not implemented.\n";
    }
    close(CF);
    $Apache::Registry::sf->disconnect();
};

# check for errors during db connection, cluck and exit
if ( $@ ne '' ) {
    my $msg = "Error during execution of database connection:\n\n$@\n";
    print << "EOF";
Content-Type: text/plain; charset=iso-8859-1
Content-Length: @{[length($msg)]}
Expires: @{[&now]}
Pragma: no-cache
Cache-Control: private

$msg
EOF
    exit(1);
}

# create output
system("$gnuplot $cf");
my $rsize = -s "$of";

# Sorry about this, but with mod_perl, Apache is doing strange things...
print << "EOF";
Content-Type: $ctype[$format]
Content-Length: @{[0+$rsize]}
Expires: @{[&now]}
Last-Modified: @{[&now]}
Cache-Control: max-age=0

EOF

# copy gnuplot result onto output stream. You cannot pass-through the
# stdout stream from any system or exec call, unless you compiled
# your Perl with AT&T's SFIO.
if ( open( IN, "<$of" ) ) {
    my $buffer;
    push @names, $of;
    syswrite(STDOUT,$buffer,$rsize) 
	while ( ($rsize=sysread(IN,$buffer,4096)) > 0 );
    close(IN);
}

# try an unlink now
unlink @names if ( @names > 0 );

#
# ---------------------------------------------------------------
#
sub ctor0 {
    # purpose: simple select from sf_peak
    # returns: gnuplot plot command, possibly more
    my $i;
#    my $out = "set logscale y\nplot ";
    my $out = 'plot ';
    for ( $i=0; $i<$n; ++$i ) {
	my $fn = "$tmpdir/$pid.$i";
	open( OUT, ">$fn" ) || die "open($fn): $!";
	push @names, $fn; # unlink after exit

	my $q = 'select st.first, ';
	$q .= $kind ? 'p.tcp_s, p.tcp_hit_s ' : 'p.tcp_r, p.tcp_hit_r ' ;
	$q .= 'from sf_stamp2 st, sf_peak p ' .
	    "where st.first between $start and $final and " .
	    "st.i2id=p.i2id and st.cache=\'$cache[$i]\' " .
	    'order by st.first';
	my $qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	my $rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	my @row;
#	print OUT $row[0], "\t", $row[1]+1, "\t", $row[2]+1, "\n"
	print OUT join("\t", @row ), "\n"
	    while ( @row = $qh->fetchrow_array );
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;

	close(OUT);
	$out .= ",\\\n " if ( $i );
	$out .= "\"$fn\" using 1:2 title \"$cache[$i]\" w l, \\\n ";
	$out .= "\"$fn\" using 1:3 title \"H-$cache[$i]\" w l";
    }
    $out;
}

#
# ---------------------------------------------------------------
#
sub ctor1 {
    # purpose: almost simple select from sf_method, all !GET methods
    # returns: gnuplot plot command, possibly more
    my $out = "set logscale y\nplot ";
#    my $out = 'plot ';
    for ( my $i=0; $i<$n; ++$i ) {
	my $fn = "$tmpdir/$pid.$i";
	open( OUT, ">$fn" ) || die "open($fn): $!";
	push @names, $fn; # unlink after exit

	my $q = 'select st.first, ';
	$q .= $kind ? 'sum(m.s), sum(m.hits) ' : 'sum(m.r), sum(m.hitr) ';
	$q .= 'from sf_stamp1 st, sf_method m ' .
	      "where st.first between $start and $final and " .
	      "st.i1id=m.i1id and st.cache=\'$cache[$i]\' and " .
	      "not ( m.method=\'GET\' or m.method=\'ICP_QUERY\' ) " .
	      'group by st.first ' .
	      'order by st.first';
	my $qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	my $rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	my @row;
#	print OUT join("\t",@row), "\n" 
	print OUT $row[0], "\t", ($row[1] or 1), "\t", ($row[2] or 1), "\n"
	    while ( @row = $qh->fetchrow_array );
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;

	close(OUT);
	$out .= ",\\\n " if ( $i );
	$out .= "\"$fn\" using 1:2 title \"$cache[$i]\" with steps, \\\n ";
	$out .= "\"$fn\" using 1:3 title \"H-$cache[$i]\" with steps";
    }
    $out;
}

#
# ---------------------------------------------------------------
#
sub ctor2 {
    # purpose: select all DFN Cache TCP traffic from sf_tcp_client
    # returns: gnuplot plot command, possibly more
    my $out = 'plot ';
    for ( my $i=0; $i<$n; ++$i ) {
	my $fn = "$tmpdir/$pid.$i";
	open( OUT, ">$fn" ) || die "open($fn): $!";
	push @names, $fn; # unlink after exit

	my $q = 'select st.first, ';
	$q .= $kind ? 'sum(c.s) ' :	'sum(c.r) ';
	$q .= 'from sf_stamp1 st, sf_tcp_client c ' .
	      "where st.first between $start and $final and " .
	      "st.i1id=c.i1id and st.cache=\'$cache[$i]\' and " .
	      "(c.host like \'%.win-ip.dfn.de\' or c.host like \'193.174.7%\'" .
	      " or c.host like \'192.168.188.%\') " .
	      'group by st.first order by st.first';
	my $qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	my $rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	my @row;
	print OUT join("\t",@row), "\n"	while ( @row = $qh->fetchrow_array );
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;

	close(OUT);
	$out .= ",\\\n " if ( $i );
	$out .= "\"$fn\" using 1:2 title \"$cache[$i]\" with steps";
    }
    $out;
}

#
# ---------------------------------------------------------------
#
sub ctor3 {
    # purpose: complex selection of IN, OUT and AMONG traffic
    #          This entails subtraction of DFN cache traffic from client.
    # returns: gnuplot plot command, possibly more
    my $out = 'plot ';
    
    # OUT from tcp_client w/o dfn caches
    my $q = 'select st.first, ';
    $q .= $kind ? 'sum(tc.s) ' : 'sum(tc.r) ';
    $q .= 'from sf_stamp1 st, sf_tcp_client tc ';
    $q .= "where st.i1id=tc.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= "and not ( tc.host like \'%.win-ip.dfn.de\' ";
    $q .= "or tc.host like \'193.174.7%\' or tc.host like \'192.168.188.%\' ) ";
    $q .= 'group by st.first';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    my %benefit;
    my @row;
    $benefit{$row[0]}[0] = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # IN from sf_hier_direct
    $q = 'select st.first, ';
    $q .= $kind ? 'sum(hd.s) ' : 'sum(hd.r) ';
    $q .= 'from sf_stamp1 st, sf_hier_direct hd ';
    $q .= "where st.i1id=hd.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= 'group by st.first';

    $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    $benefit{$row[0]}[1] = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # this is wrong!
    ## AMONG from hier_peer (siblings and parents)
    #$q = 'select st.first, ';
    #$q .= $kind ? 'sum(hp.s) ' : 'sum(hp.r) ';
    #$q .= 'from sf_stamp1 st, sf_hier_peer hp where st.i1id=hp.i1id ';
    #$q .= "and st.cache in ( $cacheset ) ";
    #$q .= "and st.first between $start and $final ";
    #$q .= 'group by st.first';
    #
    #$qh = $Apache::Registry::sf->prepare($q) or
    #	die "cannot prepare($q): $DBI::errstr";
    #$rc = $qh->execute or
    #	die "cannot execute($q): $DBI::errstr";
    #
    #$benefit{$row[0]}[3] = $row[1] while ( @row = $qh->fetchrow_array );
    #$qh->finish;

    # AMONG from tcp_client where cache is in me-group
    $q = 'select st.first, ';
    $q .= $kind ? 'sum(tc.s) ' : 'sum(tc.r) ';
    $q .= 'from sf_stamp1 st, sf_tcp_client tc ';
    $q .= "where st.i1id=tc.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= "and ( tc.host like \'%.win-ip.dfn.de\' ";
    $q .= "or tc.host like \'193.174.75.%\' or tc.host like \'193.174.75.19%\' or tc.host like \'192.168.188.%\' ) ";
    $q .= 'group by st.first';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    $benefit{$row[0]}[3] = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;

    # digest transfers - separate column, and subtract from OUT
    $q = 'select st.first, ';
    $q .= $kind ? 'sum(m.s) ' : 'sum(m.r) ';
    $q .= 'from sf_stamp1 st, sf_mime m ';
    $q .= "where st.i1id=m.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= "and m.type=\'application/cache-digest\' ";
    $q .= 'group by st.first';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    while ( @row = $qh->fetchrow_array ) {
	$benefit{$row[0]}[4] = $row[1];
	$benefit{$row[0]}[0] -= $row[1];
    }
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # SAVINGS as OUT - IN
    my $key;
    foreach $key ( keys %benefit ) {
	if ( defined $benefit{$key}[0] && defined $benefit{$key}[1] ) {
	    $benefit{$key}[2] = $benefit{$key}[0] - $benefit{$key}[1];
	    $benefit{$key}[2] = 0 if $benefit{$key}[2] < 0;
	}
    }

    # create output
    my $fn = "$tmpdir/$pid.0";
    open( OUT, ">$fn" ) || die "open($fn): $!";
    push @names, $fn; # unlink after exit

    foreach $key ( sort keys %benefit ) {
	print OUT "$key ", join(' ', map { $_+0 } @{$benefit{$key}} ), "\n";
    }

    close(OUT);
    $out .= "\"$fn\" using 1:2 title \"OUT-CD\" with steps";
    $out .= ",\\\n \"$fn\" using 1:3 title \"IN\" with steps";
    $out .= ",\\\n \"$fn\" using 1:4 title \"OUT-CD-IN\" with steps";
    $out .= ",\\\n \"$fn\" using 1:5 title \"AMONG\" with steps";
    $out .= ",\\\n \"$fn\" using 1:6 title \"CD\" with steps";
    $out;
}

#
# ---------------------------------------------------------------
#
sub subctor4 {
    # purpose: select a chosen few top level domains from sf_tld
    # paramtr: @domains (IN): domains to select
    # returns: gnuplot plot command, possibly more
    my $out = "set logscale y\nplot ";
    my @domains = @_;

    # create query
    my %benefit;
    my %sum;
    foreach my $tld ( @domains ) {
	my $q = 'select st.first, ';
	$q .= $kind ? 'sum(d.s) ' : 'sum(d.r) ';
	$q .= 'from sf_stamp1 st, sf_tld d ';
	$q .= "where st.i1id=d.i1id and st.cache in ( $cacheset ) ";
	$q .= "and st.first between $start and $final ";
	$q .= "and d.tld=\'$tld\' ";
	$q .= 'group by st.first order by st.first';

	my $qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	my $rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	my @row;
	while ( @row = $qh->fetchrow_array ) {
	    $benefit{$row[0]}{$tld} = $row[1];
	    $sum{$tld} += $row[1];
	}
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;
    }

    # create output
    my $fn = "$tmpdir/$pid.0";
    open( OUT, ">$fn" ) || die "open($fn): $!";
    push @names, $fn; # unlink after exit

    foreach my $key ( sort keys %benefit ) {
	print OUT "$key";
	foreach my $tld ( @domains ) {
	    print OUT " ", $benefit{$key}{$tld}+1; # !!! logscale !!!
	}
	print OUT "\n";
    }
    close(OUT);

    # create sort order from sums: Schwartzian Transform (perl magic)
    my @index = 
	map { $_->[0] }
	sort { $b->[1] <=> $a->[1] }
	map { [ $_ => $sum{$domains[$_]} ] } (0..$#domains);

    for ( my $n=0; $n < @domains; ++$n ) {
	$out .= ",\\\n " if $n;
	$out .= "\"$fn\" using 1:@{[$index[$n]+2]} ";
	$out .= "title \"$domains[$index[$n]]\" with steps";
    }
    $out;
}

sub ctor4 {
    # purpose: select a chosen few top level domains from sf_tld
    # returns: gnuplot plot command, possibly more
    subctor4( 'com', 'net', 'org', 'de', '<numeric>', 'edu' );
}

#
# ---------------------------------------------------------------
#
sub ctor5 {
    # purpose: select a few chosen MIME types sf_mime
    # paramtr: @mimes (IN): 
    # returns: gnuplot plot command, possibly more
    my $out = "set logscale y\nplot ";
    my @mimes = ( 'image/gif', 'image/jpeg',
		  'application/octet-stream', 'application/pdf', 
		  'text/html', 'text/plain', 'application/cache-digest',
		  'audio/*', 'video/*' );

    # create query
    my %benefit;
    my %sum;
    foreach my $type ( @mimes ) {
	my $q = 'select st.first, ';
	$q .= $kind ? 'sum(mt.s) ' : 'sum(mt.r) ';
	$q .= 'from sf_stamp1 st, sf_mime mt ';
	$q .= "where st.i1id=mt.i1id and st.cache in ( $cacheset ) ";
	$q .= "and st.first between $start and $final ";
	$q .= "and mt.type=\'$type\' ";
	$q .= 'group by st.first order by st.first';

	my $qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	my $rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	my @row;
	while ( @row=$qh->fetchrow_array ) {
	    # make this two steps, so Perl can do a String copy in the first,
	    # and lose accuracy in the second step.
	    $benefit{$row[0]}{$type} = $row[1];
	    $sum{$type} += $row[1];
	}
	    
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;
    }

    # create sort order from sums: Schwartzian Transform (perl magic)
    my @index = 
	map { $_->[0] }
	sort { $b->[1] <=> $a->[1] }
	map { [ $_ => $sum{$mimes[$_]} ] } (0..$#mimes);

    # create output
    my $fn = "$tmpdir/$pid.0";
    open( OUT, ">$fn" ) || die "open($fn): $!";
    push @names, $fn; # unlink after exit

    foreach my $key ( sort keys %benefit ) {
	print OUT "$key";
	foreach my $type ( @mimes ) {
	    print OUT " ", $benefit{$key}{$type}+1; # !!! logscale !!!
	}
	print OUT "\n";
    }
    close(OUT);

    for ( my $n=0; $n < @mimes; ++$n ) {
	$out .= ",\\\n " if $n;
	$out .= "\"$fn\" using 1:@{[$index[$n]+2]} ";
	$out .= "title \"$mimes[$index[$n]]\" with steps";
    }
    $out;
}

#
# ---------------------------------------------------------------
#
sub ctor6 {
    # purpose: select top 6 top level domains from sf_tld
    #          This is similar to ctor4, but different (dynamic domains)
    # returns: gnuplot plot command, possibly more

    # create query to determine the sums of all tlds
    my %sum;
    my $q = 'select d.tld, ';
    $q .= $kind ? 'sum(d.s) ' : 'sum(d.r) ';
    $q .= 'from sf_stamp1 st, sf_tld d ';
    $q .= "where st.i1id=d.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= 'group by d.tld';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    my @row;
    $sum{$row[0]} = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # determine the top N domains in the given interval
    subctor4( (sort { $sum{$b} <=> $sum{$a} } keys %sum)[0..5] );
}

#
# ---------------------------------------------------------------
#
sub trim ($) {
    my $x = shift;
    $x =~ s/^[ \t\r\n]+//;
    $x =~ s/[ \t\r\n]+$//;
    $x;
}

sub ctor7 {
    # purpose: select top 6 destination AS
    # returns: gnuplot plot command, possibly more
    my $out = 'plot ';

    # create query to determine the sums of all explicit AS
    my %sum;
    my $q = 'select a.asn, ';
    $q .= $kind ? 'sum(a.s) ' : 'sum(a.r) ';
    $q .= 'from sf_stamp1 st, sf_as a ';
    $q .= "where st.i1id=a.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= "and not a.asn=\'more\' ";
    $q .= 'group by a.asn';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    my @row;
    $sum{$row[0]} = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # determine the top N domains in the given interval
    my @asns = (sort { $sum{$b} <=> $sum{$a} } keys %sum)[0..5];
    my $asns = "\'" . join("\',\'",@asns) . "\'";

    $q = "select distinct asn, descr from sf_as where asn in ( $asns )";
    $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    my %names;
    $names{$row[0]} = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # now go for the values
    for ( my $n=0; $n < @asns; ++$n ) {
	$q = 'select st.first, ';
	$q .= $kind ? 'sum(a.s) ' : 'sum(a.r) ';
	$q .= 'from sf_stamp1 st, sf_as a ';
	$q .= "where st.i1id=a.i1id and st.cache in ( $cacheset ) ";
	$q .= "and st.first between $start and $final ";
	$q .= "and a.asn=\'$asns[$n]\' ";
	$q .= 'group by st.first order by st.first';

	$qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	$rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	# create output
	my $fn = "$tmpdir/$pid.$n";
	open( OUT, ">$fn" ) || die "open($fn): $!";
	push @names, $fn; # unlink after exit

	print OUT join("\t",@row), "\n"	while ( @row = $qh->fetchrow_array );
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;

	close(OUT);
	my $title = sprintf("%s %8s", trim($names{$asns[$n]}), trim($asns[$n]));
	$out .= ",\\\n " if ( $n );
	$out .= "\"$fn\" using 1:2 title \"$title\" with steps";
    }
    $out;
}

#
# ---------------------------------------------------------------
#
sub subctor8 {
    # purpose: select top N TCP clients w/ HITs
    # returns: gnuplot plot command, possibly more
    my $proto = shift;		# either 'tcp' or 'udp'
    my $what = shift;		# 0:regular, 1:hit
    my $out = 'plot ';

    # create query to determine the sums of all explicit AS
    my %sum;
    my $q = 'select c.host, ';
    $q .= $kind ? 
	( $what ? 'sum(c.hits) ' : 'sum(c.s) ' ) :
	( $what ? 'sum(c.hitr) ' : 'sum(c.r) ' );
    $q .= 'from sf_stamp1 st, sf_' . $proto . '_client c ';
    $q .= "where st.i1id=c.i1id and st.cache in ( $cacheset ) ";
    $q .= "and st.first between $start and $final ";
    $q .= "and not c.host like \'more[%]\' ";
#    $q .= "and not c.host like \'%.win-ip.dfn.de\' ";
    $q .= 'group by c.host';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    my @row;
    $sum{$row[0]} = $row[1] while ( @row = $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    # determine the top N domains in the given interval
    my @clients = (sort { $sum{$b} <=> $sum{$a} } keys %sum)[0..5];
    my $clients = "\'" . join("\',\'",@clients) . "\'";

    # now go for the values
    for ( my $n=0; $n < @clients; ++$n ) {
	$q = 'select st.first, ';
	$q .= $kind ? 
	    ( $what ? 'sum(c.hits) ' : 'sum(c.s) ' ) :
	    ( $what ? 'sum(c.hitr) ' : 'sum(c.r) ' );
	$q .= 'from sf_stamp1 st, sf_' . $proto . '_client c ';
	$q .= "where st.i1id=c.i1id and st.cache in ( $cacheset ) ";
	$q .= "and st.first between $start and $final ";
	$q .= "and c.host=\'$clients[$n]\' ";
	$q .= 'group by st.first order by st.first';

	$qh = $Apache::Registry::sf->prepare($q) or
	    die "cannot prepare($q): $DBI::errstr";
	$rc = $qh->execute or
	    die "cannot execute($q): $DBI::errstr";

	# create output
	my $fn = "$tmpdir/$pid.$n";
	open( OUT, ">$fn" ) || die "open($fn): $!";
	push @names, $fn; # unlink after exit

	print OUT $row[0], ' ', $row[1]+0, ' ', $row[2]+0, "\n"
	    while ( @row = $qh->fetchrow_array );
	$qh->finish;
	undef @row;
	undef $qh;
	undef $q;

	close(OUT);
	$out .= ",\\\n " if ( $n );
	$out .= "\"$fn\" using 1:2 title \"$clients[$n]\" with steps";
    }
    $out;
}

sub ctor8 {
    subctor8('udp',0);
}

sub ctor9 {
    subctor8('udp',1);
}

sub ctor10 {
    subctor8('tcp',0);
}

sub ctor11 {
    subctor8('tcp',1);
}

#
# ---------------------------------------------------------------
#
sub ctor12old {
    # purpose: semi-simple select from sf_peak
    # returns: gnuplot plot command, possibly more
    my $out = "set logscale y\nplot ";

    my $q = 'select st.first, ';
    $q .= $kind ? 
	'sum(p.peer_s), sum(p.parent_s), sum(p.direct_s) ' : 
	'sum(p.peer_r), sum(p.parent_r), sum(p.direct_r) ' ;
    $q .= 'from sf_stamp2 st, sf_peak p ';
    $q .= "where st.first between $start and $final and ";
    $q .= "st.i2id=p.i2id and st.cache in ( $cacheset ) ";
    $q .= 'group by st.first order by st.first';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    # open data file
    my $fn = "$tmpdir/$pid.0";
    open( OUT, ">$fn" ) || die "open($fn): $!";
    push @names, $fn; # unlink after exit

    my @row;
    print OUT join("\t",@row), "\n"
	while ( @row = map { $_+1 } $qh->fetchrow_array );
    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    close(OUT);
    $out .= "\"$fn\" using 1:2 title \"SIBLING\" w l, \\\n ";
    $out .= "\"$fn\" using 1:3 title \"PARENT\" w l, \\\n ";
    $out .= "\"$fn\" using 1:4 title \"DIRECT\" w l";
    $out;
}

#
# ---------------------------------------------------------------
#
sub ctor12 {
    # purpose: semi-simple select from sf_peak
    # returns: gnuplot plot command, possibly more
    my $out = "set logscale y\nplot ";

    my $q = 'select st.first, ';
    $q .= $kind ? 
	'sum(p.peer_s), sum(p.parent_s), sum(p.direct_s) ' : 
	'sum(p.peer_r), sum(p.parent_r), sum(p.direct_r) ' ;
    $q .= 'from sf_stamp2 st, sf_peak p ';
    $q .= "where st.first between $start and $final and ";
    $q .= "st.i2id=p.i2id and st.cache in ( $cacheset ) ";
    $q .= 'group by st.first order by st.first';

    my $qh = $Apache::Registry::sf->prepare($q) or
	die "cannot prepare($q): $DBI::errstr";
    my $rc = $qh->execute or
	die "cannot execute($q): $DBI::errstr";

    # open data file
    my $fn = "$tmpdir/$pid.0";
    open( O0, ">$fn" ) || die "open($fn): $!";
    push @names, $fn; # unlink after exit
    $fn = "$tmpdir/$pid.1";
    open( O1, ">$fn" ) || die "open($fn): $!";
    push @names, $fn; # unlink after exit
    $fn = "$tmpdir/$pid.2";
    open( O2, ">$fn" ) || die "open($fn): $!";
    push @names, $fn; # unlink after exit

    my @row;
    while ( @row = $qh->fetchrow_array ) {
	print O0 $row[0], "\t", $row[1], "\n" if $row[1] > 0;
	print O1 $row[0], "\t", $row[2], "\n" if $row[2] > 0;
	print O2 $row[0], "\t", $row[3], "\n" if $row[3] > 0;
    }

    $qh->finish;
    undef @row;
    undef $qh;
    undef $q;

    close(O0);
    close(O1);
    close(O2);

    $out .= "\"$names[-3]\" using 1:2 title \"SIBLING\" w l, \\\n ";
    $out .= "\"$names[-2]\" using 1:2 title \"PARENT\" w l, \\\n ";
    $out .= "\"$names[-1]\" using 1:2 title \"DIRECT\" w l";
    $out;
}
